/****************************************************************************/
/* 				Marcu Alina - Elena											*
*					Grupa 331 CA 											*
*				Proiect PM - Grizzle, The Grumpy Bear						*/
/***************************************************************************/
#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>
#include "diskio.h"
#include "pff.h"

#define FCC(c1,c2,c3,c4)	(((DWORD)c4<<24)+((DWORD)c3<<16)+((WORD)c2<<8)+(BYTE)c1)	/* FourCC */

/*---------------------------------------------------------*/

volatile BYTE FifoRi, FifoWi, FifoCt;							/* FIFO controls */

unsigned char Buff1[256];										/* Wave output FIFO */
unsigned char Buff2[256];										/* Wave output FIFO */

FATFS Fs;														/* File system object */
DIR Dir,
	Dir_laugh,
	Dir_talk,
	Dir_sing,
	Dir_grumpy;

DWORD count_laugh, count_sing, count_talk, count_grumpy;
BOOL fin_laugh, fin_sing, fin_talk, fin_grumpy;	
								/* Directory object */
FILINFO Fno;													/* File information */
BOOL b1_pressed, /* PA5 = belly_button */
	 b2_pressed, /* PA6 = right_button */
	 b3_pressed; /* PA7 = left_button */								
BOOL s1_pressed, /* PA2 = nose_switch */
	 s2_pressed, /* PA3 = right_switch */
	 s3_pressed; /* PA4 = left_switch */
WORD rb;														/* Return value. Put this here to avoid bugs of avr-gcc */

int state, joyLevel, tickle, pinch, start, end, wait1, wait2;

static DWORD load_header (void) {
	DWORD sz;

	if (pf_read(Buff1, 12, &rb))
		return 1;												/* Load file header (12 bytes) */

	if (rb != 12 || LD_DWORD(Buff1+8) != FCC('W','A','V','E'))
		return 0;

	for (;;) {
		pf_read(Buff1, 8, &rb);									/* Get Chunk ID and size */
		if (rb != 8) return 0;
		
		sz = LD_DWORD(&Buff1[4]);								/* Chunk size */

		switch (LD_DWORD(&Buff1[0])) 
		{							/* FCC */
			case FCC('f','m','t',' ') :								/* 'fmt ' chunk */
				if (sz > 100 || sz < 16) 
					return 0;										/* Check chunk size */
				pf_read(Buff1, sz, &rb);							/* Get content */
				if (rb != sz) 
					return 0;
				if (Buff1[0] != 1)
					return 0;										/* Check coding type (1) */
				if (Buff1[2] != 1 && Buff1[2] != 2) 				/* Check channels (1/2) */
					return 0;
			
				if (Buff1[14] != 8 && Buff1[14] != 16)				/* Check resolution (8/16) */
					return 0;
			
				OCR0A = (BYTE)(F_CPU/8/LD_WORD(&Buff1[4]))-1;		/* Sampling freq */
			
			break;

			case FCC('d','a','t','a') :								/* 'data' chunk (start to play) */
				return sz;

			case FCC('L','I','S','T') :								/* 'LIST' chunk (skip) */
			case FCC('f','a','c','t') :								/* 'fact' chunk (skip) */
				pf_lseek(Fs.fptr + sz);
				break;

			default :												/* Unknown chunk (error) */
				return 0;
		}
	}

	/* 0:Invalid format, 1:I/O error, >1:Number of samples */
	return 0;
}

volatile uint8_t read1 = 1;

volatile uint16_t counter = 0;

volatile uint8_t sync = 0;

ISR(TIMER0_COMPA_vect) 
{
	if (sync == 1) 
	{
		if (read1 == 1) 
		{
			OCR1A = (unsigned char)Buff1[counter++];
			if(counter == 256) 
			{
				read1 = 0;
				counter = 0;
			}
		}
		else if (read1 == 0) 
		{
			OCR1A = (unsigned char)Buff2[counter++];
			if (counter == 256) 
			{
				read1 = 1;
				counter = 0;
			}
		}
	}
}

void Timer0_init(void) 
{
	TCCR0A = _BV(WGM01);
	TCCR0B = _BV(CS01);											/* CTC, FCPU/8 */
	TIMSK0 = _BV(OCIE0A);
}

void Timer1_init(void) 
{
	TCCR1A = _BV(COM1A1)|_BV(COM1A0)|_BV(WGM10);
	TCCR1B = _BV(WGM12)|_BV(CS10);
}

void reset_state(void) 
{
	b1_pressed = FALSE;
	b2_pressed = FALSE;
	b3_pressed = FALSE;
	
	s1_pressed = FALSE;
	s2_pressed = FALSE;
	s3_pressed = FALSE;
	
	state = 0;
}

static UINT play (const char *fn)
{
	DWORD sz;
	FRESULT res;
	unsigned char read2;

	
	uint16_t size = 256;
	
	if ((res = pf_open(fn)) == FR_OK) {
	
		Timer0_init();
		Timer1_init();
		
		sz = load_header();			/* Load file header */
		if (sz < 256) return (UINT)sz;
		
		
		FifoCt = 0; FifoRi = 0; FifoWi = 0;		/* Reset FIFO */
		pf_read(Buff1, 512 - (Fs.fptr % 512), &rb);	/* Snip sector unaligned part */
		sz -= rb;

			
		res = pf_read(Buff1, size, &rb);
		if (res != FR_OK) {;while (FifoCt) ; return res; }
		
		sync = 1;
				
		do {
			read2 = read1;
			
			if(read1 == 1)	
				res = pf_read(Buff2, size, &rb);
			else
				res = pf_read(Buff1, size, &rb);
				
			if (res != FR_OK) { break; }
			
			while(read2 == read1);
			
			
			
		
		} while (rb == size);	/* Repeat until all data read */
		
		sync = 0;
	}
	while (FifoCt) ;			/* Wait for FIFO empty */
	

	return res;
}

/* void playWAV (void) {
	
	unsigned char filename[20];
	
	while (!pf_readdir(&Dir, &Fno) && Fno.fname[0]) // go through all the files in the current folder 
	{
		if (!(Fno.fattrib & (AM_DIR|AM_HID))) // filter out all the folders, hidden files and the files that are not "WAV" 
		{
			// play the WAV file (the play(filename) function needs the full name of the file e.g. music/file.wav) 
			//sprintf(filename, "sing/%s", Fno.fname);
			if (strstr(filename, ".WAV") != NULL) 
			{
				PORTA = PORTA ^ (0x01 << 3);
				play(filename);
				_delay_ms(500);
				PORTA &= ~(0x01 << 3);
				_delay_ms(500);
			}
		}
	}
}*/

void check_joy ()
{
	if (!(PINA & (1 << PA5)))
	{
		b1_pressed = TRUE;
	}
	
	if (!(PINA & (1 << PA6)))
	{
		b2_pressed = TRUE;
	}
	
	if (!(PINA & (1 << PA7)))
	{
		b3_pressed = TRUE;
	}
	
	if (b1_pressed == TRUE || b2_pressed == TRUE || b3_pressed == TRUE)
	{
		joyLevel++;
	}
	
	reset_state ();
}

void grizzle (void)
{
	switch (state)
	{
		case 0:
			if ( ((PINA & (1 << PA2)) == 0) || ((PINA & (1 << PA3)) == 0) || ((PINA & (1 << PA4)) == 0) || 
				 ((PINA & (1 << PA5)) == 0) || ((PINA & (1 << PA6)) == 0) || ((PINA & (1 << PA7)) == 0) )
				{
					reset_state();
					_delay_ms(100);
					state = 1;
					wait1 = 0;
				}
		break;
		
		case 1:
			play ("Grizzle/Hello.wav");
			
			if (!(PINA & (1 << PA6)))
			{
				b2_pressed = TRUE;
			}
			
			else if (!(PINA & (1 << PA7)))
			{
				b3_pressed = TRUE;
			}
			
			if (b2_pressed == TRUE || b3_pressed == TRUE)
			{
				state = 2;
				reset_state ();
			}
		break;
		
		case 2:
			play ("Grizzle/IamGrumpy.wav");
			check_joy ();
			state = 3;
		break;
	}
	
}

int main()
{
	/* Set pins and ports */
	DDRA |= (1 << PA0) | (1 << PA1);
	DDRA &= ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTA |= (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7);

	DDRB |= (1 << PB4) | (1 << PB5) | (1 << PB7);
	PORTB &= ~(1 << PB6);

	DDRC = 0xFF;
	DDRD = 0xFF;

	//PORTA ^= ~(0x01 << PA0);
	//PORTA ^= ~(0x01 << PA1);
	
	sei();
	
	/* no states */
	reset_state();
	
	for (;;) {
		_delay_ms(500);								/* Delay 500ms for things to settle down */
		PORTD ^= ~(0x01 << 6);
		if (pf_mount(&Fs)) {	
			continue;								/* Initialize FS */
		}
		
		if (pf_opendir(&Dir, "Grizzle"))
		{
			PORTA = PORTA ^ (0x01 << PA0);
			break;
		}
			
	
		while(1) 
		{
			grizzle ();
		}
		
		//reset_state();
	}
	
		
	return 1;
}